#include <FL/Fl.H>
#include <FL/Fl_Menu_Button.H>
#include <FL/Fl_Menu_Item.H>
//#include <FL/Fl_PNG_Image.H>
#include <FL/Fl_Single_Window.H>
#include <FL/x.H>

#include <iostream>
#include <stdio.h>

class nobox_Fl_Menu_Button : public Fl_Menu_Button
{
public:
  nobox_Fl_Menu_Button(int X, int Y, int W, int H)
    : Fl_Menu_Button(X, Y, W, H) { }

protected:
  void draw() {
    draw_box(FL_NO_BOX, color());
    draw_label();
  }
};

static Fl_Single_Window *win;
static nobox_Fl_Menu_Button *but;
static Fl_RGB_Image *rgb = NULL;
//static const char *icon = NULL;

static void callback(Fl_Widget *, void *);
static void close_cb(Fl_Widget *, void *);

static Fl_Menu_Item menu_items[] = {
  { "Launch", 0, callback, 0,0, FL_NORMAL_LABEL, 0, 14, 0 },
  { "Close",  0, close_cb, 0,0, FL_NORMAL_LABEL, 0, 14, 0 },
  { 0,0,0,0,0,0,0,0,0 }
};

static const char *tooltip = "Click me";

static void set_size(void *)
{
  int w = win->w();
  int h = win->h();

  //if (w == 0 && h == 0) {
  if (h == 0) {
    /* repeat until the window was resized to the tray's size */
    Fl::repeat_timeout(0.001, set_size);
    return;
  }

  if (w > h) {
    h = w;
  } else {
    w = h;
  }

  win->size(w, h);

  if (h > 256) {
    w = h = 256;
  } else if (h > 128) {
    w = h = 128;
  } else if (h > 96) {
    w = h = 96;
  } else if (h > 64) {
    w = h = 64;
  } else if (h > 48) {
    w = h = 48;
  } else if (h > 32) {
    w = h = 32;
  } else if (h > 22) {
    w = h = 22;
  } else {
    w = h = 16;
  }

 /*
  if (icon && strlen(icon) > 0) {
    rgb = new Fl_PNG_Image(icon);
  }

  if (!rgb) {
    rgb = new Fl_PNG_Image(NULL, src_icon_png, src_icon_png_len);
  }

  but->image(rgb->copy(w, h));
  */
  but->size(w, h);
  win->redraw();

  if (rgb) {
    delete rgb;
  }
}

static void callback(Fl_Widget *, void *) {
  win->hide();
  std::cout << "Callback!" << std::endl;
}

static void close_cb(Fl_Widget *, void *) {
  win->hide();
  std::cout << "closed" << std::endl;
}

static int create_tray_entry(void)
{
  Fl_Color flcol = 0;
  Window dock;
  XColor xcol;
  XImage *image;
  XEvent ev;
  char atom_tray_name[128];

  win = new Fl_Single_Window(0, 0, 0, 0);
  but = new nobox_Fl_Menu_Button(0, 0, 0, 0);
  but->menu(menu_items);
  but->tooltip(tooltip);
  win->end();
  win->clear_border();
  win->show();
  if (!win->shown()) {
    return 1;
  }

  snprintf(atom_tray_name, sizeof(atom_tray_name), "_NET_SYSTEM_TRAY_S%i", fl_screen);
  dock = XGetSelectionOwner(fl_display, XInternAtom(fl_display, atom_tray_name, False));
  if (!dock) {
    return 1;
  }

  memset(&ev, 0, sizeof(ev));
  ev.xclient.type = ClientMessage;
  ev.xclient.window = dock;
  ev.xclient.message_type = XInternAtom(fl_display, "_NET_SYSTEM_TRAY_OPCODE", False);
  ev.xclient.format = 32;
  ev.xclient.data.l[0] = fl_event_time;
  ev.xclient.data.l[1] = 0;  // SYSTEM_TRAY_REQUEST_DOCK
  ev.xclient.data.l[2] = fl_xid(win);
  ev.xclient.data.l[3] = 0;
  ev.xclient.data.l[4] = 0;
  XSendEvent(fl_display, dock, False, NoEventMask, &ev);
  XSync(fl_display, False);

  /* get tray color and set as background */
  image = XGetImage(fl_display, RootWindow(fl_display, DefaultScreen(fl_display)),
                    win->x() + 1, win->y() + 1, 1, 1, AllPlanes, XYPixmap);
  xcol.pixel = XGetPixel(image, 0, 0);
  XFree(image);

  XQueryColor(fl_display, DefaultColormap(fl_display, DefaultScreen(fl_display)), &xcol);
  Fl::set_color(flcol, xcol.red >> 8, xcol.green >> 8, xcol.blue >> 8);
  win->color(flcol);
  win->redraw();

  Fl::add_timeout(0.005, set_size);

  return Fl::run();
}

int main(void)
{
  if (create_tray_entry() != 0) {
    std::cerr << "error: cannot create tray/indicator entry" << std::endl;
    return 1;
  }

  return 0;
}

